/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import { Injectable } from '@angular/core';
import { Observable, Subject } from 'rxjs';

export const EventMap = {
  LayoutChangedEvent: 'LAYOUT_CHANGED_EVENT',
  WindowResizedEvent: 'WINDOW_RESIZED_EVENT',
};

interface EventRegistry {
  [eventName: string]: Subject<any>;
}

@Injectable({
  providedIn: 'root',
})
export class EventBusService {
  private eventRegistry: EventRegistry = {};

  constructor() {}

  getEvent(eventName: string): Observable<any> {
    if (!this.eventRegistry[eventName]) {
      this.eventRegistry[eventName] = new Subject<any>();
    }

    const subject = this.eventRegistry[eventName];
    return subject.asObservable();
  }

  publish(eventName: string, data?: any) {
    const subject = this.eventRegistry[eventName];

    if (subject) {
      subject.next(data);
    }
  }
}
